# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('story', '0010_image_story'),
    ]

    operations = [
        migrations.AddField(
            model_name='content',
            name='audio',
            field=models.FileField(blank=True, upload_to='content/%Y/%m/', null=True),
        ),
        migrations.AlterField(
            model_name='image',
            name='image',
            field=models.ImageField(blank=True, upload_to='content/%Y/%m/', null=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('story', '0012_auto_20160526_0244'),
    ]

    operations = [
        migrations.AlterField(
            model_name='story',
            name='starting_scene',
            field=models.ForeignKey(related_name='starting', on_delete=django.db.models.deletion.SET_NULL, blank=True, to='story.Scene', null=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('story', '0004_auto_20151226_0807'),
    ]

    operations = [
        migrations.CreateModel(
            name='Consequence',
            fields=[
                ('id', models.AutoField(serialize=False, auto_created=True, primary_key=True, verbose_name='ID')),
                ('module', models.CharField(max_length=255)),
                ('choice', models.ForeignKey(to='story.Choice', on_delete=models.CASCADE)),
            ],
        ),
        migrations.CreateModel(
            name='ConsequenceAttribute',
            fields=[
                ('id', models.AutoField(serialize=False, auto_created=True, primary_key=True, verbose_name='ID')),
                ('tag', models.CharField(max_length=50)),
                ('value', models.SmallIntegerField(default=1)),
                ('choice', models.ForeignKey(to='story.Choice', on_delete=models.CASCADE)),
            ],
        ),
        migrations.CreateModel(
            name='RequiredTag',
            fields=[
                ('id', models.AutoField(serialize=False, auto_created=True, primary_key=True, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('tag', models.CharField(max_length=50)),
                ('comparison', models.CharField(choices=[('>', '>'), ('<', '<'), ('=', '=')], max_length=10, default='>')),
                ('value', models.SmallIntegerField(default=1)),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType', on_delete=models.CASCADE)),
            ],
        ),
        migrations.AddField(
            model_name='content',
            name='group',
            field=models.CharField(max_length=50, default='default'),
        ),
        migrations.AddField(
            model_name='content',
            name='ordering',
            field=models.SmallIntegerField(default=0),
        ),
    ]



# -*- coding: utf-8 -*-

from zipfile import ZipFile

from django import forms
from django.contrib import admin, messages
from django.contrib.contenttypes.admin import GenericTabularInline
from django.core.cache import cache
from django.core.files.base import ContentFile
from django.shortcuts import get_object_or_404, redirect, render, resolve_url
from django.template import RequestContext
from six import BytesIO, StringIO

from .models import Choice, Content, Image, RequiredTag, Scene, Story

try:
    from docx import Document
except ImportError:
    Document = None


class ContentInline(admin.TabularInline):
    model = Content
    extra = 0

    def get_formset(self, request, instance, **kwargs):

        class ContentForm(forms.ModelForm):
            class Meta(object):
                model = Content
                exclude = []

        return super(ContentInline, self).get_formset(
            request, instance, form=ContentForm, **kwargs)


class ChoiceInline(admin.TabularInline):
    model = Choice
    extra = 0
    fk_name = "scene"

    readonly_fields = ("admin_link",)

    def admin_link(self, obj):
        return "<a href='{}'>{}</a>".format(
            resolve_url("admin:story_scene_change", obj.next_scene.pk),
            obj.next_scene
        )
    admin_link.short_description = ""
    admin_link.allow_tags = True

    def get_formset(self, request, instance, **kwargs):

        class ChoiceForm(forms.ModelForm):
            class Meta(object):
                model = Choice
                exclude = []
            next_scene = forms.ModelChoiceField(
                label="next",
                queryset=Scene.objects.filter(story=instance.story))

        return super(ChoiceInline, self).get_formset(
            request, instance, form=ChoiceForm, **kwargs)


class SceneAdmin(admin.ModelAdmin):
    inlines = (ContentInline, ChoiceInline)
    search_fields = ("name", )
    list_display = ("name", "story")
    list_filter = ("story", )

    readonly_fields = ("admin_links_to",)

    def admin_links_to(self, obj):
        links = []
        for link in obj.links_to.all():
            links.append(
                "<a href='{}'>{}</a>".format(
                    resolve_url("admin:story_scene_change", link.scene.pk),
                    link
                )
            )
        return "<br>".join(links)
    admin_links_to.allow_tags = True
    admin_links_to.short_description = "Links to this scene"


class RequiredTagInline(GenericTabularInline):
    model = RequiredTag
    extra = 0


class ContentAdmin(admin.ModelAdmin):
    search_fields = ("text", )
    inlines = (RequiredTagInline,)


class ChoiceAdmin(admin.ModelAdmin):
    search_fields = ("text", )
    inlines = (RequiredTagInline,)


class StoryImportForm(forms.Form):
    docx_file = forms.FileField()


class SceneInline(admin.TabularInline):
    model = Scene
    extra = 0

    exclude = ("image",)
    readonly_fields = ("link",)

    def link(self, obj):
        return "<a href='../../scene/{}/'>edit</a>".format(obj.id)
    link.allow_tags = True


class StoryAdmin(admin.ModelAdmin):
    search_fields = ("name", )
    comparisons = (
        (":", ">="),
        ("==", "="),
        ("=", "="),
        ("<<", "<<"),
        (">>", ">>"),
        (">", ">"),
        ("<", "<"),
        (">=", ">="),
        ("<=", "<="),
    )

    inlines = [SceneInline]

    def _strip_tags(self, text):

        text = text.strip()

        retval = []
        if text.startswith("["):
            tags, text = text.split("]", 1)
            tags = tags[1:].split(",")
            for tag in tags:
                for operator, compare in self.comparisons:
                    if operator in tag:
                        name, value = tag.split(operator)
                        name = name.strip()
                        value = value.strip()
                        comparison = compare
                        try:
                            value = int(value)
                        except ValueError:
                            pass
                        retval.append((name, comparison, value))
                        break

        return text, retval

    def _get_image(self, story, name):
        try:
            return Image.objects.get(story=story, name=name)
        except Image.DoesNotExist:
            return None

    def import_audio(self, request, pk, *args, **kwargs):
        try:
            class AudioForm(forms.Form):
                zip_file = forms.FileField()

            if request.method == "POST":
                form = AudioForm(request.POST, files=request.FILES)
                if form.is_valid():
                    zip = ZipFile(form.cleaned_data['zip_file'])

                    for path in zip.namelist():
                        scene_name, filename = path.split("/", 1)
                        if filename:
                            content_index, _ = filename.split(".", 1)
                            content_index = int(content_index)

                            # match Scene and content
                            try:
                                scene = Scene.objects.get(
                                    story__pk=pk,
                                    name__iexact=scene_name,
                                )
                                content = Content.objects.filter(
                                    scene__story__pk=pk,
                                    scene__name__iexact=scene_name
                                )[content_index - 1]
                                # print(content, filename)
                                content.audio.save(
                                    filename,
                                    ContentFile(zip.read(path)),
                                    save=True
                                )
                            except Scene.DoesNotExist:
                                print("Failed to find scene: {} {} {}".format(
                                    pk, scene_name, Scene.objects.filter(
                                        story__pk=pk
                                    ).values_list(
                                        "name", flat=True
                                    )
                                ))
                            except (IndexError, Content.DoesNotExist):
                                print("Failed to find content: {} {} {}".format(
                                    scene_name,
                                    content_index,
                                    scene.content_set.count()
                                ))

                    messages.success(request, "audio imported")
                    return redirect("../../" + pk + "/")
            else:
                form = AudioForm()
        except TypeError as ex:
            form = AudioForm()

        return render(
            request,
            "story/admin_import_audio.html",
            dict(form=form),
        )

    def import_scenes(self, request, pk, *args, **kwargs):
        if not pk:
            messages.success(
                request, "Create the story first, then import scenes")
            return redirect("..")

        instance = get_object_or_404(Story, pk=pk)
        self.story = instance
        self.scene_defaults = {}
        errors = []
        p = ""
        try:
            form = None
            error = None
            cache.set(instance.cache_key, None)
            if request.method == "POST":
                form = StoryImportForm(request.POST, request.FILES)
                scenes = []
                if form.is_valid():
                    filelike = BytesIO(form.cleaned_data["docx_file"].read())
                    doc = Document(filelike)
                    mode = None
                    scene = None
                    content = None
                    choice = None
                    first = True
                    links = []
                    for p in doc.paragraphs:
                        style = p.style.style_id
                        if style == "Heading1":
                            if p.text.strip():
                                instance.name = p.text
                                instance.save()
                                mode = "title"
                        elif style == "Heading2":
                            if p.text.strip():
                                mode = "content"
                                scene = Scene.objects.get_or_create(
                                    story=instance,
                                    name=p.text.strip(),
                                )[0]
                                if self.scene_defaults:
                                    for name, value in self.scene_defaults.items():
                                        setattr(self.scene, name, value)
                                    self.scene.save()
                                self.scene = scene
                                if first:
                                    instance.starting_scene = scene
                                    instance.save()
                                    first = False
                                scenes.append(scene)
                                scene.content_set.all().delete()
                                scene.links_from.all().delete()
                        elif style == "Heading3":
                            # if p.text.strip().lower().startswith("choice"):
                            mode = "choices"
                        elif style == "ListParagraph" or style == "TextBody" or style == "Normal":
                            if p.text.strip():

                                if "=>" not in p.text:
                                    text = ""
                                    for run in p.runs:
                                        if run.bold:
                                            text += u"**{}**".format(run.text)
                                        elif run.italic:
                                            text += u"*{}*".format(run.text)
                                        else:
                                            text += run.text
                                    text, tags = self._strip_tags(text)
                                    text = text.strip()

                                    if text:
                                        content = scene.content_set.create(
                                            text=text.strip()
                                        )
                                        for name, comp, value in tags:
                                            if name == "background":
                                                self.scene.image = self._get_image(story=self.story, name=value)
                                                self.scene.save()
                                                self.scene_defaults['image'] = self.scene.image
                                            else:
                                                print("creating tag", name, value, comp)
                                                content.tags.create(
                                                    tag=name,
                                                    value=value,
                                                    comparison=comp,
                                                )
                                        #
                                        # self.run_content_processors(tags)

                                else:
                                    text, consequences = p.text.split("=>")
                                    text, tags = self._strip_tags(text)
                                    choice = scene.links_from.create(
                                        text=text
                                    )
                                    for name, comparison, value in tags:
                                        choice.tags.create(
                                            tag=name,
                                            value=value,
                                            comparison=comparison,
                                        )
                                    for consequence in consequences.split(","):
                                        consequence = consequence.strip()
                                        if "+" in consequence:
                                            name, value = consequence.split("+", 1)
                                            choice.consequenceattribute_set.create(
                                                tag=name.strip(),
                                                value=int(value)
                                            )
                                        elif "-" in consequence:
                                            name, value = consequence.split("-", 1)
                                            choice.consequenceattribute_set.create(
                                                tag=name.strip(),
                                                value=-int(value)
                                            )
                                        else:
                                            links.append((choice, consequence))
                        else:
                            print(" ! Missing", p.style.style_id, p.text)

                    for choice, next_scene_name in links:
                        # print("linking choice: {} to scene: {}".format(
                        #     choice, next_scene_name
                        # ))
                        try:
                            if next_scene_name.strip():
                                choice.next_scene = Scene.objects.get(
                                    story=instance,
                                    name__iexact=next_scene_name.strip()
                                )
                                choice.save()
                        except Scene.MultipleObjectsReturned:
                            print("!!! More than one scene named: {}".format(
                                next_scene_name))
                        except Scene.DoesNotExist:
                            print("!!! Can't find next scene named: {}".format(
                                next_scene_name))
                    # TODO: report unused scenes

            else:
                form = StoryImportForm()
        except Exception as ex:
            import traceback
            error = u"{}: \n\n{}\n\n*** From Paragraph: {}".format(
                ex, traceback.format_exc(), p and p.text or "")
            print(error)
            errors.append(error)

        response = render(
            request,
            "story/admin_import.html",
            dict(
                form=form,
                instance=instance,
                errors=errors,
            )
        )

        return response


class ImageAdmin(admin.ModelAdmin):
    search_fields = ("name",)
    list_display = ("name", "story", "image")
    list_filter = ("story",)


admin.site.register(Story, StoryAdmin)
admin.site.register(Scene, SceneAdmin)
admin.site.register(Content, ContentAdmin)
admin.site.register(Choice, ChoiceAdmin)
admin.site.register(Image, ImageAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('story', '0007_story_starting_scene'),
    ]

    operations = [
        migrations.AddField(
            model_name='scene',
            name='image',
            field=models.ImageField(null=True, upload_to=b'scene/%Y/%m/', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('story', '0006_auto_20151230_1717'),
    ]

    operations = [
        migrations.AddField(
            model_name='story',
            name='starting_scene',
            field=models.ForeignKey(related_name='starting', blank=True, to='story.Scene', null=True, on_delete=models.CASCADE),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('story', '0003_auto_20151226_0805'),
    ]

    operations = [
        migrations.AlterField(
            model_name='scene',
            name='story',
            field=models.ForeignKey(to='story.Story', default=1, on_delete=models.CASCADE),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('story', '0005_auto_20151228_1934'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='content',
            options={'verbose_name_plural': 'Content', 'ordering': ('ordering',)},
        ),
        migrations.AlterModelOptions(
            name='story',
            options={'verbose_name_plural': 'Stories'},
        ),
        migrations.AddField(
            model_name='content',
            name='image',
            field=models.ImageField(upload_to='content/%Y/%m/', blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='requiredtag',
            name='comparison',
            field=models.CharField(default='>=', choices=[('>', '>'), ('<', '<'), ('=', '=')], max_length=10),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('story', '0002_auto_20151226_0750'),
    ]

    operations = [
        migrations.CreateModel(
            name='Story',
            fields=[
                ('id', models.AutoField(auto_created=True, verbose_name='ID', serialize=False, primary_key=True)),
                ('name', models.CharField(max_length=255)),
            ],
        ),
        migrations.AddField(
            model_name='scene',
            name='story',
            field=models.ForeignKey(null=True, blank=True, to='story.Story', on_delete=models.CASCADE),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('story', '0013_auto_20160528_0528'),
    ]

    operations = [
        migrations.AlterField(
            model_name='requiredtag',
            name='value',
            field=models.CharField(max_length=50),
        ),
        migrations.AlterField(
            model_name='story',
            name='short_description',
            field=models.CharField(max_length=80, null=True, blank=True),
        ),
    ]



import base64
import json

from django.conf import settings
from django.contrib.contenttypes.fields import (GenericForeignKey,
                                                GenericRelation)
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.core.cache import cache
from django.db import models
from django.utils.safestring import mark_safe

import markdown


class Image(models.Model):
    story = models.ForeignKey("Story", on_delete=models.CASCADE)
    name = models.CharField(max_length=50)
    image = models.ImageField(
        upload_to="content/%Y/%m/", blank=True, null=True)

    def __str__(self):
        return self.name

    @property
    def full_url(self):
        site = Site.objects.get_current()
        if self.image.url.startswith("http"):
            return self.image.url
        else:
            return "http://{}{}".format(site.domain, self.image.url)


class Story(models.Model):
    ordering = models.SmallIntegerField(default=1)
    name = models.CharField(max_length=255)
    short_description = models.CharField(blank=True, null=True, max_length=80)
    splash = models.ImageField(upload_to="content/%Y/%m/", blank=True, null=True)

    starting_scene = models.ForeignKey(
        "Scene", null=True, blank=True, related_name="starting",
        on_delete=models.SET_NULL)

    class Meta:
        verbose_name_plural = "Stories"
        ordering = ("ordering", "name")

    def __str__(self):
        return self.name

    @property
    def cache_key(self):
        return "story-2-{}".format(self.id)

    def to_dict(self):
        site = Site.objects.get_current()
        splash = None
        try:
            splash = "http://{}{}".format(site.domain, self.splash.url)
        except ValueError:
            pass
        dataset = dict(
            id=self.pk,
            meta=dict(
                name=self.name,
                short_description=self.short_description,
                splash=splash,
                start=self.starting_scene_id,
            ),
            media=[i.full_url for i in self.image_set.all()]
        )
        scenes = []
        for scene in self.scene_set.order_by("id"):
            scenes.append(scene.to_dict())
        dataset['scenes'] = scenes
        return dataset

    def to_json(self):
        # dataset = cache.get(self.cache_key)
        # if settings.DEBUG or not dataset:
        dataset = self.to_dict()
        # cache.set(self.cache_key, dataset, 60 * 60 * 4)
        return json.dumps(dataset)


class Scene(models.Model):
    story = models.ForeignKey(Story, on_delete=models.CASCADE)
    name = models.CharField(max_length=255, blank=True, null=True)
    image = models.ForeignKey(
        Image, verbose_name="Background Image", null=True, blank=True, on_delete=models.CASCADE)

    def __str__(self):
        return u"{name}".format(
            story=self.story,
            name=self.name or u"#{}".format(self.id))

    def to_dict(self):
        from .content_processors import TransformParagraph
        tp = TransformParagraph(self.story)
        return dict(
            meta=dict(
                id=self.id,
                name=self.name
            ),
            image=self.image and self.image.full_url or None,
            content=[c.to_dict(tp) for c in self.content_set.all()],
            choices=[c.to_dict() for c in self.links_from.all()],
        )


class RequiredTag(models.Model):
    object_id = models.PositiveIntegerField()
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    content_object = GenericForeignKey()

    tag = models.CharField(max_length=50)
    comparison = models.CharField(max_length=10, choices=(
        (">", ">"),
        ("<", "<"),
        ("=", "=")
    ), default=">=")
    value = models.CharField(max_length=50)

    def __str__(self):
        return self.tag


class Content(models.Model):
    scene = models.ForeignKey(Scene, on_delete=models.CASCADE)
    ordering = models.SmallIntegerField(default=0)

    # grouping content will make only one show, whichever has the highest
    # ordering and matches the character tags
    group = models.CharField(max_length=50, default="default")
    tags = GenericRelation(RequiredTag)
    text = models.TextField()
    audio = models.FileField(upload_to="content/%Y/%m/", null=True, blank=True)

    class Meta:
        verbose_name_plural = "Content"
        ordering = ("ordering", "id")

    def __str__(self):
        return u"{}: {}".format(self.scene, self.text[:25] + u"...")

    @property
    def formatted_text(self):

        return mark_safe(markdown.markdown(self.text))

    @property
    def image_data_url(self):
        if self.image:
            return "data:image/png;base64,{}".format(
                base64.b64encode(self.image.image.read())
            )
        return None

    def to_dict(self, tp):
        if not tp:
            tp = TransformParagraph(self.scene.story)

        tags, text = tp.run_content_processors(
            list(self.tags.values_list("tag", "comparison", "value")),
            self.text
        )

        audio = None
        if self.audio:
            site = Site.objects.get_current()
            if self.audio.url.startswith("http"):
                audio = self.audio.url
            else:
                audio = "http://{}{}".format(site.domain, self.audio.url)

        return dict(
            text=text,
            audio=audio,
            tags=[dict(tag=n, comparison=c, value=v) for n, c, v in tags]
        )


class Choice(models.Model):
    scene = models.ForeignKey(Scene, related_name="links_from", on_delete=models.CASCADE)
    text = models.CharField(max_length=255)
    next_scene = models.ForeignKey(
        Scene, related_name="links_to", blank=True, null=True, on_delete=models.CASCADE)
    tags = GenericRelation(RequiredTag)

    def __str__(self):
        return u"{}: {}".format(self.scene, self.text[:25] + u"...")

    @property
    def formatted_text(self):
        tp = TransformParagraph(self.scene.story)

        tags, text = tp.run_content_processors(
            list(self.tags.values_list("tag", "comparison", "value")),
            self.text
        )

        # strip off the paragraph tags
        return text[3:-4]

    def to_dict(self):
        return dict(
            text=self.formatted_text,
            next_scene=self.next_scene_id,
            tags=list(self.tags.values("tag", "comparison", "value")),
            consequences=list(self.consequenceattribute_set.values(
                "tag", "value"
            ))
        )

    # def link(self):
    #     if self.next_scene:
    #         return resolve_url


class Consequence(models.Model):
    choice = models.ForeignKey(Choice, on_delete=models.CASCADE)
    module = models.CharField(max_length=255)


class ConsequenceAttribute(models.Model):
    choice = models.ForeignKey(Choice, on_delete=models.CASCADE)
    tag = models.CharField(max_length=50)
    value = models.SmallIntegerField(default=1)


# this requires Image
from .content_processors import TransformParagraph  # isort:skip



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('story', '0009_auto_20160227_2125'),
    ]

    operations = [
        migrations.AddField(
            model_name='image',
            name='story',
            field=models.ForeignKey(default=2, to='story.Story', on_delete=models.CASCADE),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('story', '0008_scene_image'),
    ]

    operations = [
        migrations.CreateModel(
            name='Image',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=50)),
                ('image', models.ImageField(null=True, upload_to=b'content/%Y/%m/', blank=True)),
            ],
        ),
        migrations.RemoveField(
            model_name='content',
            name='image',
        ),
        migrations.RemoveField(
            model_name='scene',
            name='image',
        ),
        migrations.AddField(
            model_name='content',
            name='image',
            field=models.ForeignKey(blank=True, to='story.Image', null=True, on_delete=models.CASCADE),
        ),
        migrations.AddField(
            model_name='scene',
            name='image',
            field=models.ForeignKey(blank=True, to='story.Image', null=True, on_delete=models.CASCADE),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('story', '0011_auto_20160509_0258'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='content',
            name='image',
        ),
        migrations.AddField(
            model_name='story',
            name='short_description',
            field=models.TextField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='story',
            name='splash',
            field=models.ImageField(null=True, upload_to=b'content/%Y/%m/', blank=True),
        ),
        migrations.AlterField(
            model_name='choice',
            name='text',
            field=models.CharField(max_length=255),
        ),
        migrations.AlterField(
            model_name='scene',
            name='image',
            field=models.ForeignKey(verbose_name=b'Background Image', blank=True, to='story.Image', null=True, on_delete=models.CASCADE),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('story', '0014_auto_20160604_2311'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='content',
            options={'ordering': ('ordering', 'id'), 'verbose_name_plural': 'Content'},
        ),
        migrations.AlterModelOptions(
            name='story',
            options={'ordering': ('ordering', 'name'), 'verbose_name_plural': 'Stories'},
        ),
        migrations.AddField(
            model_name='story',
            name='ordering',
            field=models.SmallIntegerField(default=1),
        ),
    ]



import json

from django.core.management.base import BaseCommand, CommandError

from story.models import Story


class Command(BaseCommand):
    help = 'Exports a story for the cordova app'

    def handle(self, *args, **options):
        stories = []
        for story in Story.objects.all():
            stories.append(story.to_dict())

        with open("client/www/stories.json", "w") as fh:
            fh.write(json.dumps(stories))



# -*- coding: utf-8 -*-

import collections

from django.utils.safestring import mark_safe

import markdown



class TransformParagraph(object):

    def __init__(self, story):
        self.story = story
        self.content_defaults = collections.OrderedDict()

    def run_content_processors(self, tags, text):
        ntags = collections.OrderedDict()
        unused = []
        for name, comp, value in tags:
            if comp in ("<<", ">>"):
                ntags[name] = (name, comp, value)
            else:
                unused.append((name, comp, value))

        for name, value in self.content_defaults.items():
            ntags.setdefault(name, value)

        images = {}
        for image in self.story.image_set.all():
            images["images/" + image.name] = "<img src='{}' alt=''>".format(
                image.full_url,
                image.name
            )

        text = text.format(**images)

        for name, comp, value in ntags.values():
            try:
                func = getattr(self, "content_" + name, None)
                if func:
                    text = func(text, name, comp, value)
                else:
                    unused.append((name, comp, value))
            except Exception as ex:
                import traceback
                traceback.print_exc()
                print("EXCEPTION:", ex)

        return (
            unused,
            mark_safe(markdown.markdown(text.strip()).strip())
        )

    def get_image_url(self, name):
        try:
            return Image.objects.get(story=self.story, name=name).full_url
        except Image.DoesNotExist:
            return ""

    def startswith_any(self, s, chars):
        return any([s.startswith(c) for c in chars])

    def endswith_any(self, s, chars):
        return any([s.endswith(c) for c in chars])

    ## content processores ###################################################

    def content_char(self, text, name, comp, value):

        if self.startswith_any(text, ['"', u"“"]) and \
                self.endswith_any(text, ['"', u"”"]):
            text = self.content_quote(text[1:-1], name, comp, value)
        elif self.startswith_any(text, ['<']) and \
                self.endswith_any(text, ['>']):
            text = self.content_thought(text[1:-1], name, comp, value)
        else:
            text = self.content_narration(text, name, comp, value)

        text = u"""<p class="plain"><image src="{image}" class="{direction}"></p>
{text}""".format(
            text=text,
            image=self.get_image_url(value),
            direction=comp == ">>" and "right" or "left"
        )

        self.content_defaults[name] = (name, comp, value)

        return text

    def content_quote(self, text, name, comp, value):
        return u"""<blockquote class="{direction}">{text}</blockquote>""".format(
            text=text,
            direction=comp == ">>" and "right" or "left"
        )

    def content_thought(self, text, name, comp, value):
        return u"""<blockquote class="thought {direction}">{text}</blockquote>""".format(
            text=text,
            direction=comp == ">>" and "right" or "left"
        )

    def content_narration(self, text, name, comp, value):
        return u"""<p class="{direction}">{text}</p>""".format(
            text=text,
            direction={
                ">>": "right",
                "<<": "left",
            }.get(comp, "")
        )

    def content_image(self, text, name, comp, value):
        return u"""<p class="plain"><image src="{image}"></p>\n
{text}""".format(
            text=text,
            image=self.get_image_url(value),
        )

from .models import Image



from setuptools import setup, find_packages

setup(
    name='django-story-builder',
    version='0.2.11',
    author='John Leith',
    author_email='leith.john@gmail.com',
    packages=find_packages(),
    url='https://gitlab.com/leith-john/django-story-builder',
    description='Django project for creating CYOAs',
    install_requires=[
        "markdown", "python-docx"
    ],
    include_package_data=True,
    zip_safe=False,
)



from django.views.generic.base import TemplateView

from .models import Story


class StoryView(TemplateView):
    template_name = "story/index.html"

    def get_context_data(self, **kwargs):
        return super(StoryView, self).get_context_data(
            story=Story.objects.get(pk=self.args[0]),
            **kwargs
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Choice',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', auto_created=True, primary_key=True)),
                ('text', models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name='Content',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', auto_created=True, primary_key=True)),
                ('text', models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name='Scene',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=255, blank=True, null=True)),
            ],
        ),
        migrations.AddField(
            model_name='content',
            name='scene',
            field=models.ForeignKey(to='story.Scene', on_delete=models.CASCADE),
        ),
        migrations.AddField(
            model_name='choice',
            name='next_scene',
            field=models.ForeignKey(related_name='links_to', to='story.Scene', on_delete=models.CASCADE),
        ),
        migrations.AddField(
            model_name='choice',
            name='scene',
            field=models.ForeignKey(related_name='links_from', to='story.Scene', on_delete=models.CASCADE),
        ),
    ]
